from tkinter import *
import math
#
# ■相対描画のためのRelativeDrawクラス定義
#　  
#   変数名 = RelativeDraw(canvas, startX, startY,scaleX, scaleY, lineWidth,color)
#
#       canvas          : キャンバス
#       startX, startY  : 表示開始位置
#       scaleX, scaleY  : 表示スケール(上下または左右を逆転するときは負の値で指定)
#       lineWidth       : 描画する直線の太さ
#       color           : カラーを文字列で指定(例 : 'red', '#00FFFF')
#
class RelativeDraw:
    #■ コンストラクタ
    def __init__(self, canvas,startX, startY,scaleX, scaleY, lineWidth,color):
        self.canvas=canvas
        self.X0=startX
        self.Y0=startY
        self.SX=scaleX
        self.SY=scaleY
        self.lineWidth=lineWidth
        self.color=color
        self.goto(0,0)
        
    #■ XY座標設定
    def goto(self,x,y):
        self.x=x
        self.y=y
    #■ カラー設定
    def setColor(self,color):
        self.color=color
    #■ 線幅設定
    def setWidth(self,lineWidth):
        self.lineWidth=lineWidth
        
    #■ スクリーン座標への変換
    def screenX(self, X):
        return self.X0 + self.SX *  X
    def screenY(self, Y):
        return self.Y0 + self.SY *  Y

class TurtleGraph(RelativeDraw):
    #■ コンストラクタ
    def __init__(self, canvas,startX, startY,scaleX, scaleY, lineWidth,color):
        RelativeDraw.__init__(self,canvas,startX, startY,scaleX, scaleY,
                              lineWidth,color)
        self.setAng(0)
    #■ 角度設定
    def setAng(self,th):
        self.th=th

    #■ 向きの変更(左回転は負の値で指定)
    def turn(self,th):
        self.th=self.th+th

    #■ 前に進む(後退は負の値で指定)
    def forward(self,L):
        TH=self.th*math.pi/180
        X=self.x+L*math.cos(TH)
        Y=self.y+L*math.sin(TH)
        self.canvas.create_line(self.screenX(self.x), self.screenY(self.y),
                                self.screenX(X)     , self.screenY(Y)     ,
                                width = self.lineWidth, fill = self.color)
        self.x=X
        self.y=Y


#■コッホ曲線のクラス定義(curve以外はTurtleGraph, RelativeDrawクラスからの継承)
class Koch(TurtleGraph):
    def curve(self,N,L,DA):
        if N<=0:
            self.forward(L)
        else:
            NN=N-1; LL=L/3
            L2=LL/(2*math.cos(DA*math.pi/180))
            self.curve(NN,LL,DA)
            self.turn(DA)
            self.curve(NN,L2,DA)
            self.turn(-2*DA)
            self.curve(NN,L2,DA)
            self.turn(DA)
            self.curve(NN,LL,DA)

#■ tkinter，Canvas共通処理
def initTk(Title):
    tk=Tk()
    tk.title(Title)
    return tk
def initCanvas(W,H):
    canvas=Canvas(tk,width=W,height=H)
    canvas.pack()
    return canvas

#■　メイン処理
tk=initTk("Koch雪片")#Koch snowflake
canvas=initCanvas(220,250)
koch=Koch(canvas, 10, 60, 1, -1, 2, 'blue')
DT=60
MX=205
NL=5
CTab=["#FF0000","#FF7700","#FFFF00","#77FF00","#00FF00","#007700",
      "#007777","#0077FF","#0000FF","#7700FF","#770077","#FF0077",]
koch.setWidth(1)

for DT in range(-60,60):
    koch.setColor(CTab[int(abs(DT)/4) % 12])
    for k in range(3):
        koch.setAng(-120*k);koch.curve(NL,MX,DT) 
    tk.update()
koch.setColor('#770000')
for k in range(3):
        koch.setAng(-120*k);koch.curve(NL,MX,60) 
tk.update()

tk.mainloop()
